#include "all03.h"


// Tesla MH74S571
// Abwandlung des Programms EEP3.c durch Christian Matenar 2009
// Details zum MH74S571:
// Programmierspannung ist 21V!
//
// Schritte: 
// - _CE Sinal auf 5V
// - 5V muss an pin 16 (Vcc) angelegt werden
// - 21V muss an pin 16 (Vcc) angelegt werden
// - Adresse muss mit TTL Pegeln angelegt werden
// - Daten: 	Es darf immer nur ein Bit gebrannt werden
//				Das zu brennende Bitt muss auf 0V liegen, 
//				alle anderen bits mit 3.9kohm auf 5V
// - 21V muss an pin 16 (Vcc) angelegt werden
// - _CE Signal auf 0V -> jetzt wird gebrannt.
// - _CE Signal auf 5V.
// - 5V muss an pin 16 (Vcc) angelegt werden
// - 0V muss an pin 16 (Vcc) angelegt werden
//
// Timing: 	Siehe do_program( long addr, unsigned char val )
//			(Zeiten wurden via Oszi berprft, es funktioniert auf 
//			(meinem P90 Computer)

//1.4 13: A7		VCC	 :28 3.3
//1.5 14: A6		A8	 :27 3.2
//1.6 15: A5		A9	 :26 3.1
//1.7 16: A4		/CE	 :25 3.0
//2.0 17: A1		D0	 :24 2.7
//2.1 18: A2		D1	 :23 2.6
//2.2 19: A3		D2	 :22 2.5
//    20: GND		D3	 :21 2.4


#define _CE		25

static char apin[] = { 17,18,19,16,15,14,13,27,26 };
static char dpin[] = { 24,23,22,21 };

struct DEV {
	char name[20];
	long Size;
	char VCC;
};

struct DEV TESLA_devs[] = {
	{ "MH74S571",	0x200L, 50 },
};


struct {
	char name[20];
	int numdevs;
	struct DEV *dev;
} mfr[] = {
	{ "TESLA", sizeof(TESLA_devs)/sizeof(struct DEV), TESLA_devs }
};

int mfrno = 0, devno = 0;

long DevStart= 0L;
long Counter = 0L;
long lastaddr;

void ShowCounter( long val )
{
	struct text_info ti;

	gettextinfo( &ti );

	textattr( ( BLUE << 4 ) + WHITE );
	locate( 8,71 ); cprintf( " %05lX ", val );

	textattr( ti.attribute );
	gotoxy( ti.curx, ti.cury );
}

void power( int state )
{
	int i;

	// we dont need these...
	setdac( VHHID, 0 );					// VHH = 0V
	setdac( VOPID, 0 );					// VOP = 0V

	setport( OTHERENID, 0, 0 );			// Pin20 = GND

	setport( VHHENCID,0, 0 );			// no VHHC
	setport( VHHENCID,1, 0 );			// no VHHC

	setport( VCCENID, 0, 0 );			// no VCC
	setport( VCCENID, 1, 0 );			// no VCC

	for( i = 0; i <= 4; ++i )
	{
		setport( VOPENID, i, 0 );		// no VOP
		setport( VHHENID, i, 0 );		// no VHH
		setport( TTLID,   i, 0xFF );	// TTL hi on all pins
	}

	lastaddr = -1L;

	// VCC is pin 16 (programmer socket pin 28)

	if( state )
	{
		setdac( VCCID, mfr[mfrno].dev[devno].VCC );	// VCC = 5.0V
		waitms( 100 );
		setpin( 28, VCCENID, 1 );
	
		//cm
//		setdac( VOPID, 110+6 );			// VOP = 11V	
	setdac( VOPID, 200+6 );			// VOP = 20V		
		waitms( 100 );

		
	}
	else
	{
		setdac( VCCID, 0 );				// VCC = 0V
		waitms( 100 );
		setpin( 28, VCCENID, 0 );
		
		//cm
		setdac( VOPID, 0 );				// VOP = 0V
		waitms( 100 );
	
	}
}

void setaddr( unsigned long addr )
{
	register unsigned char i;
	register unsigned a, b;

	a = (unsigned)( addr & 0xFFFF );
	b = (unsigned)( lastaddr & 0xFFFF );

	for( i = 0; i < 9; ++i )		// A0..A9
	{
		if( ( a & 1 ) != ( b & 1 ) )
			setpin( apin[i], TTLID, a & 1 );
		a >>= 1; b >>= 1;
	}



	lastaddr = addr;
}



//setup adr bus & data bits
void f_write( long addr, register unsigned char d )
{
	register int i;

	setaddr( addr );

	for( i = 0; i < 4; ++i )
	{
		setpin( dpin[i], TTLID, d & 1 );
		d >>= 1;
	}

//	setpin( _CE, TTLID, 0 );	//hmm?
//	setpin( _CE, TTLID, 1 );	//hmm?
}

void f_prepare( void )
{
	int i;

	for( i = 0; i < 4; ++i )
		setpin( dpin[i], TTLID, 1 );	// set D0..D3 to 1 for reading
}


// read data from dpins
unsigned char f_read( void )
{
	register unsigned char d = 0;
	register int i;


	setpin( _CE, TTLID, 0 );
	

	for( i = 3; i >= 0; --i )
		d = ( d << 1 ) | getpin( dpin[i] );


	setpin( _CE, TTLID, 1 );
	
//CM	
waitms( 30 );	

	return d;
}



int do_program( long addr, unsigned char val )
{
	unsigned char d,v;
	int i;

	d=1;
	for( i = 3; i >= 0; --i )
	{
		v=(val&d);
				
		locate( 1,62 ); cprintf( "val:       %i", val );
		locate( 2,62 ); cprintf( "v:         %i", v );
		
		if (v!=0)					// muss gebrannt werden?
		{
		 f_write( addr, (v^255) );			// adr.bus und daten anlegen (invers mit ttl pegeln)
						
		 setpin( 28, VCCENID, 1 );	// normale VCC an
		 us_delay(10);				// t7 (geraten)
		 setpin( _CE, TTLID, 1 );	// Chip deaktiv
		 us_delay(10);				// t5
		 setpin( 28, VOPENID, 1 );	// hohe Spannung an
		 us_delay(500);				// t1
		 setpin( _CE, TTLID, 0 );	// Chip aktiv
		 waitms( 4 );				// X (erst 2)
		 setpin( _CE, TTLID, 1 );	// Chip deaktiv
		 us_delay(500);				// t2=t1
		 setpin( 28, VOPENID, 0 );	// hohe Spannung aus
		 setpin( 28, VCCENID, 1 );	// normale VCC an
		 us_delay(10);
		 us_delay(1000);
	
		 setpin( _CE, TTLID, 0 );	// Chip aktiv
		 us_delay(10); 				// t7 (geraten)
		 waitms( 1 );

		 setpin( 28, VCCENID, 0 );	// normale VCC aus
		}	
		waitms( 14 );				// t6=3y (erst 7)	
		d = ( d << 1 ) ;
	}
	

	

	return 1;
		
	//waitms( 10 );
}


void ShowType( void )
{
	if( mfrno < 0 || mfrno > sizeof(mfr) / sizeof(mfr[0]) )
		mfrno = 0;
	if( devno < 0 || devno >= mfr[mfrno].numdevs )
		devno = 0;

	bufsize = mfr[mfrno].dev[devno].Size;
	BufEnd = bufsize - 1;

	textattr( ( BLUE << 4 ) | WHITE );

	locate( 0,40 ); cprintf( "*Mfr.: %s", mfr[mfrno].name );
	locate( 1,40 ); cprintf( "*TYPE: %s", mfr[mfrno].dev[devno].name );

	locate( 0,62 ); cprintf( "*PGMSPEED: TESLA" );
	locate( 1,62 ); cprintf( "*VCC :%3.1fV", 0.1 * mfr[mfrno].dev[devno].VCC );
	locate( 2,62 ); cprintf( "*Vprog :%3.1fV", 21.0 );
	locate( 6,41 ); cprintf( "       end   addr.: %05lX", BufEnd );

	textattr( ( CYAN << 4 ) | WHITE );
}


void bprom_program( void )
{
	int done;
	long addr;
	FILE *f;

	textattr( ( CYAN << 4 ) | WHITE );
	_window( 11,40, 23,79 );

	textattr( ( BLUE << 4 ) | WHITE );
	locate( 11, 45 ); cprintf( "   PROGRAM :" );

	for(;;)
	{
		textattr( ( CYAN << 4 ) | WHITE );
		locate( 12, 41 ); cprintf( "Ready to program (Y/<CR>)? " );

		for( done = 0; !done; )
		{
			switch( getch() )
			{
			case 0:
				getch();
				break;

			case '\n':
			case '\r':
			case 0x1B:
				return;

			case 'y':
			case 'Y':
				done = 1;
			}
		}

		clscrn( 13,41, 22,78 );

		setport( USERBITS, 0, 0 );
		power(1);
	
		
		if( !buffer && ( f = fopen( buffile, "rb" ) ) == NULL )
		{
			errbeep();
			textattr( ( RED << 4 ) | WHITE );
			locate( 13, 41 ); cprintf( "File open error !" );
			textattr( ( CYAN << 4 ) | WHITE );
			delay(1000);
			break;
		}

		textattr( BLINK | ( LIGHTGREEN << 4 ) | WHITE );
		locate( 13, 41 ); cprintf( "Programming now... " );

		ShowCounter( BufStart );
		for( addr = BufStart; addr <= BufEnd; ++addr )
		{
			if( ( addr & 0x0F ) == 0 )
				ShowCounter( addr );

			if( !do_program( addr, ( buffer ? buffer[addr] : getc(f) ) ) )
				break;
		}

		if( !buffer ) fclose( f );

		textattr( ( CYAN << 4 ) | WHITE );
		locate( 13, 41 ); cprintf( "Programming now... " );

		locate( 14, 41 );
		if( addr > BufEnd )
		{
			ShowCounter( BufEnd );
			putchar( 7 );
			setport( USERBITS, 0, 8 );
			cprintf( " OK !" );
		}
		else
		{
			errbeep();
			textattr( ( RED << 4 ) | WHITE );
			cprintf( "Program error ! at %05lX", addr );
			textattr( ( CYAN << 4 ) | WHITE );
		}

		power(0);
	
	}
}

void bprom_read( void )
{
	int done;
	long addr;
	unsigned char val;
	FILE *f;

	textattr( ( CYAN << 4 ) | WHITE );
	_window( 11,40, 23,79 );

	textattr( ( BLUE << 4 ) | WHITE );
	locate( 11, 45 ); cprintf( "   READ to buffer :" );

	for(;;)
	{
		textattr( ( CYAN << 4 ) | WHITE );
		locate( 12, 41 ); cprintf( "Ready to start (Y/Even/Odd/<CR>)? " );

		for( done = 0; !done; )
		{
			switch( getch() )
			{
			case 0:
				getch();
				break;

			case '\n':
			case '\r':
			case 0x1B:
				return;

			case 'y':
			case 'Y':
				done = 1;

			case 'e':
			case 'E':
			case 'o':
			case 'O':
				done = 1;
			}
		}

		clscrn( 13,41, 22,78 );

		setport( USERBITS, 0, 0 );
		power(1);

		if( !buffer && ( f = fopen( buffile, "wb" ) ) == NULL )
		{
			errbeep();
			textattr( ( RED << 4 ) | WHITE );
			locate( 13, 41 ); cprintf( "File open error !" );
			textattr( ( CYAN << 4 ) | WHITE );
			delay(1000);
			break;
		}

		textattr( BLINK | ( LIGHTGREEN << 4 ) | WHITE );
		locate( 13, 41 ); cprintf( "Reading now... " );

		Chks = 0;

		ShowCounter( BufStart );
		for( addr = BufStart; addr <= BufEnd; ++addr )
		{
			if( ( addr & 0x0F ) == 0 )
				ShowCounter( addr );

			setaddr( addr );

			Chks += ( val = f_read() );

			if( buffer ) buffer[ addr ] = val;
			else putc( val, f );
		}
		ShowCounter( BufEnd );

		textattr( ( BLUE << 4 ) | WHITE );
		locate( 7,41 ); cprintf( "       Check Sum  :  %04X", Chks );

		if( !buffer ) fclose( f );

		textattr( ( CYAN << 4 ) | WHITE );
		locate( 13, 41 ); cprintf( "Reading now... " );

		locate( 14, 41 );
		putchar( 7 );
		cprintf( " OK !" );

		power(0);
	}
}

void bprom_verify( void )
{
	int done;
	long addr;
	FILE *f;

	textattr( ( CYAN << 4 ) | WHITE );
	_window( 11,40, 23,79 );

	textattr( ( BLUE << 4 ) | WHITE );
	locate( 11, 45 ); cprintf( "   VERIFY with buffer :" );

	for(;;)
	{
		textattr( ( CYAN << 4 ) | WHITE );
		locate( 12, 41 ); cprintf( "Ready to verify (Y/Even/Odd/<CR>)? " );

		for( done = 0; !done; )
		{
			switch( getch() )
			{
			case 0:
				getch();
				break;

			case '\n':
			case '\r':
			case 0x1B:
				return;

			case 'y':
			case 'Y':
				done = 1;

			case 'e':
			case 'E':
			case 'o':
			case 'O':
				done = 1;
			}
		}

		clscrn( 13,41, 22,78 );

		setport( USERBITS, 0, 0 );
		power(1);

		if( !buffer && ( f = fopen( buffile, "rb" ) ) == NULL )
		{
			errbeep();
			textattr( ( RED << 4 ) | WHITE );
			locate( 13, 41 ); cprintf( "File open error !" );
			textattr( ( CYAN << 4 ) | WHITE );
			delay(1000);
			break;
		}

		textattr( BLINK | ( LIGHTGREEN << 4 ) | WHITE );
		locate( 13, 41 ); cprintf( "Verifying now... " );

		ShowCounter( BufStart );
		for( addr = BufStart; addr <= BufEnd; ++addr )
		{
			if( ( addr & 0x0F ) == 0 )
				ShowCounter( addr );

			setaddr( addr );

			if( ( ( buffer ? buffer[addr] : getc(f) ) & 0xFF ) != f_read() )
				break;
		}

		if( !buffer ) fclose( f );

		textattr( ( CYAN << 4 ) | WHITE );
		locate( 13, 41 ); cprintf( "Verifying now... " );

		locate( 14, 41 );
		if( addr > BufEnd )
		{
			ShowCounter( BufEnd );
			putchar( 7 );
			cprintf( " OK !" );
			setport( USERBITS, 0, 8 );
		}
		else
		{
			errbeep();
			textattr( ( RED << 4 ) | WHITE );
			cprintf( " VERIFY ERROR ! at %05lX", addr );
			textattr( ( CYAN << 4 ) | WHITE );
		}

		power(0);
	}
}

void type_select( void )
{
	int done, i;
	char no[10];

	textattr( ( CYAN << 4 ) | WHITE );
	_window( 11,40, 23,79 );

	textattr( ( BLUE << 4 ) | WHITE );
	locate( 11, 45 ); cprintf( "  TYPE SELECT:" );

	textattr( ( CYAN << 4 ) | WHITE );

	for( i = 0; i < mfr[mfrno].numdevs; ++i )
	{
		locate( 12+i, 41 ); cprintf( "%d.%s", i, mfr[mfrno].dev[i].name );
	}

	locate( 21, 41 ); cprintf( "<CR> back to main menu." );
	locate( 22, 41 ); cprintf( "SELECT NUMBER ?" );

	for(;;)
	{
		for( no[0] = done = 0; !done; )
		{
			locate( 22, 56 ); cprintf( "%s ", no );
			locate( 22, 56+strlen(no) );

			switch( i = getch() )
			{
			case 0:
				getch();
				break;

			case 8:
				if( ( i = strlen(no) ) != 0 )
					no[i-1] = 0;
				break;

			case '\n':
			case '\r':
				if( strlen( no ) &&
					( i = atoi(no) ) >= 0 && i < mfr[mfrno].numdevs )
				{
					devno = i;
					ShowType();
					return;
				}
				break;

			case 0x1B:
				return;

			default:
				if( isdigit( i ) )
					strcat( no, (char*)&i );
				break;
			}
		}
	}
}

void mfr_select( void )
{
	int done, i;
	char no[10];

	textattr( ( CYAN << 4 ) | WHITE );
	_window( 11,40, 23,79 );

	textattr( ( BLUE << 4 ) | WHITE );
	locate( 11, 45 ); cprintf( "  MFR SELECT:" );

	textattr( ( CYAN << 4 ) | WHITE );

	for( i = 0; i < sizeof(mfr) / sizeof(mfr[0]); ++i )
	{
		locate( 12+i, 41 ); cprintf( "%d.%s", i, mfr[i].name );
	}

	locate( 21, 41 ); cprintf( "<CR> back to main menu." );
	locate( 22, 41 ); cprintf( "SELECT NUMBER ?" );

	for(;;)
	{
		for( no[0] = done = 0; !done; )
		{
			locate( 22, 56 ); cprintf( "%s ", no );
			locate( 22, 56+strlen(no) );

			switch( i = getch() )
			{
			case 0:
				getch();
				break;

			case 8:
				if( ( i = strlen(no) ) != 0 )
					no[i-1] = 0;
				break;

			case '\n':
			case '\r':
				if( strlen( no ) &&
					( i = atoi(no) ) >= 0 && i < sizeof(mfr) / sizeof(mfr[0]) )
				{
					mfrno = i;
					if( devno >= mfr[mfrno].numdevs )
						devno = 0;
					ShowType();
					return;
				}
				break;

			case 0x1B:
				return;

			default:
				if( isdigit( i ) )
					strcat( no, (char*)&i );
				break;
			}
		}
	}
}

/*=========================================================*/
int main( void )
{
	int ch = 0, redraw, first = 1;

	/*---------------------------------------------------------*/
	/* main program starts here                                */
	/*---------------------------------------------------------*/

	getcwd( oldpath, 260 );
	strcpy( path, oldpath );

	ReadConfig();

	for( ;; )
	{
		if( first )
		{
			first = 0;
			
			init_hw();
			initdacs();
			setport( USERBITS, 0, 0 );

#ifdef WIN32
			if( !buffer ) buffer = farmalloc( bufsize );
			else buffer = farrealloc( buffer, bufsize );
#else
			if( !buffer ) buffer = (void huge *)farmalloc( bufsize );
			else buffer = (void huge*)farrealloc( (void far*)buffer, bufsize );
#endif
		}

		textattr( ( LIGHTGRAY << 4 ) | YELLOW ); clscrn( 0,0, 24,79 );

		locate( 0,0 ); cprintf( "Universal   Programmer" );
		locate( 1,0 ); cprintf( "MODEL: PC Based" );
		locate( 2,0 ); cprintf( "BPROM section V1.00" );

		textattr( ( BLUE << 4 ) + WHITE ); clscrn( 0,40, 8,79 );

		ShowType();
		power(0);

		textattr( ( BLUE << 4 ) + WHITE ); _window( 3,40, 9,79 );
		locate( 4,41 ); cprintf( "             TARGET ZONE" );
		locate( 5,41 ); cprintf( "Buffer start addr.: %05lX (BYTE WIDE)", BufStart );
		locate( 6,41 ); cprintf( "       end   addr.: %05lX", BufEnd );
		locate( 7,41 ); cprintf( "       Check Sum  :  %04X", Chks );
		locate( 8,41 ); cprintf( "Device start addr.: %05lX", DevStart );

		_window( 6,69, 9,79 );
		locate( 7,71 ); cprintf( "COUNTER" );

		ShowCounter( 0L );

		textattr( ( CYAN << 4 ) | WHITE ); clscrn( 3,0, 23,38 );

		locate( 3,0 ); cprintf( "-------------  Main Menu  -------------" );
		locate( 4,0 ); cprintf( "1. DOS SHELL                           " );
		locate( 5,0 ); cprintf( "2. Load BIN or HEX file to buffer      " );
		locate( 6,0 ); cprintf( "3. Save buffer to disk                 " );
		locate( 7,0 ); cprintf( "4. Edit buffer                         " );
		locate( 8,0 ); cprintf( "5. Change I/O base address             " );
		locate( 9,0 ); cprintf( "                                       " );
		locate(10,0 ); cprintf( "7. Display buffer                      " );
		locate(11,0 ); cprintf( "M. Mfr. select                         " );
		locate(12,0 ); cprintf( "T. Type select                         " );
		locate(13,0 ); cprintf( "                                       " );
		locate(14,0 ); cprintf( "                                       " );
		locate(15,0 ); cprintf( "R. Read                                " );
		locate(16,0 ); cprintf( "P. Program                             " );
		locate(17,0 ); cprintf( "V. Verify                              " );
		locate(18,0 ); cprintf( "                                       " );
		locate(19,0 ); cprintf( "                                       " );
		locate(20,0 ); cprintf( "Q. Quit                                " );
		locate(21,0 ); cprintf( "---------------------------------------" );
		locate(22,0 ); cprintf( "Buffer size      : %ld bytes", bufsize  );
		locate(23,0 ); cprintf( "Buffer structure : %s", buffile );


		for( redraw = 0; !redraw; )
		{
			textattr( ( BLUE << 4 ) + WHITE ); clscrn( 24,0, 24,38 );

			locate( 24,0 ); cprintf( "Select function ? " );

			for(;;)
			{
				if( ( ch = getch() ) != 0 )
					break;
				getch();						 /* neglect extended code */
			}

			switch( ch = toupper(ch) )
			{
			case '1': dos_shell( "" ); redraw = 1; break;
			case '2': load_file(); redraw = 1; break;
			case '3': save_file(); break;
			case '4': edit_buffer(); redraw = 1; break;
			case '5': set_io_adr(); redraw = first = 1; break;
			case '7': edit_buffer(); redraw = 1; break;		
			case 'M': mfr_select(); redraw = first = 1; break;
			case 'T': type_select(); redraw = first = 1; break;

			case 'B': ; break;
			case 'E': ; break;
			case 'P': bprom_program(); break;
			case 'V': bprom_verify(); break;
			case 'A': ; break;
			case 'R': bprom_read(); break;

			case '\n':
			case '\r': redraw = 1; break;	// refresh
			}

			setport( USERBITS, 0, 0 );

			if( ch == 'Q' )
			{
				WriteConfig();
				textattr( LIGHTGRAY ); clrscr();
				chdir( oldpath );
				return( 0 );
			}

			textattr( ( LIGHTGRAY << 4 ) | YELLOW ); clscrn( 10,40, 23,79 );
		}
	}
}
